# Bundestag2017Forecast
# simulate the turnout as well as the forecasting model to generate valid
# measures of uncertainty around prediction

# rm(list=ls())

library(foreign)
library(lme4)
library(dplyr)
library(nlme)

setwd('your/directory/here')

d <- read.dta('master.dta') %>% tbl_df()

nsim <- 500

# Turnout ----------------------------------------------------------------------

t <- read.dta('turnout_data.dta') %>% tbl_df()

# model estimated on the original data
m_t <- lmer(turnout ~ (1 + elec_year | state_id) + l_turnout + elec_year +
              state_id, t)

# simulation
m_t_sim <- arm::sim(m_t, nsim)

X <- t %>% mutate(intercept = 1) %>%
  select(intercept, l_turnout, elec_year, state_id) %>%
  mutate(state_id = as.character(state_id)) %>%
  filter(elec_year == 2017)

mm <- model.matrix(~ state_id, data = X)

X <- cbind(X, mm[, -1])


predictions_t <- data.frame()
# cycle through countries to include country-specific intercepts & effects
for(s in unique(X$state_id)) {
  coef <- fixef(m_t_sim)

  coef[, 1] <- coef[, 1] + ranef(m_t_sim)$state_id[, s, 1]
  coef[, 3] <- coef[, 3] + ranef(m_t_sim)$state_id[, s, 2]

  tmp <- (coef %*% t(as.matrix(X[which(X$state_id == s),c(1:3, 5:19)])) +
    rnorm(nsim, mean = 0, sd = attr(VarCorr(m_t), "sc"))) %>%
  data.frame(1:nsim, s, .)
  names(tmp) <- c('simid_t', 'state', 'turnout_pred')
  predictions_t <- bind_rows(predictions_t, tmp)
}
predictions_t <- tbl_df(predictions_t)


predictions_t %>% group_by(state) %>% summarise(mean = mean(turnout_pred),
                                              lwr = quantile(turnout_pred, 0.025),
                                              upr = quantile(turnout_pred, .975))

tmp <-
  d %>% filter(elec_year == 2017, state != 'Bundesgebiet', party == 'spd') %>%
  select(state, electorate)

# add electorate numbers
predictions_t <- left_join(predictions_t, tmp, by = 'state')

# calculate the number of valid votes for 2017
predictions_t$valid <- predictions_t$turnout_pred * predictions_t$electorate

# Unweighted model
# Vote shares (state level) ----------------------------------------------------

# Model estimated on the original data -
summary(m_v <- lmer(voteshare ~ (1 + voteshare_state | state) +
                      (1 | party:state) +
                      voteshare_l + voteshare_state + pm_party + gpya_l +
                      pm_partyXgpya_l + pmyears + pmyearsXpm_party, d,
                    subset = state != 'Bundesgebiet', REML = F)
)

# simulation
m_v_sim <- arm::sim(m_v, nsim)

X <- d %>% mutate(intercept = 1) %>%
  select(state, party, elec_year, intercept, voteshare_l, voteshare_state,
         pm_party, gpya_l, pm_partyXgpya_l, pmyears, pmyearsXpm_party) %>%
  filter(elec_year == 2017, state != 'Bundesgebiet')

predictions_v <- data.frame()
# cycle through countries to include country-specific intercepts & effects
for(i in 1:nrow(X)) {
  s <- as.character(X[i, 'state'])
  p <- as.character(X[i, 'party'])

  coef <- fixef(m_v_sim)
  coef[, 1] <- coef[, 1] +
    ranef(m_v_sim)$`party:state`[, paste(p, s, sep = ':'), 1] +
    ranef(m_v_sim)$state[, s, 1]
  coef[, 3] <- coef[, 3] +
    # ranef(m_v_sim)$`party:state`[, paste(p, s, sep = ':'), 2] +
    ranef(m_v_sim)$state[, s, 2]

  tmp <- (coef %*% t(as.matrix(X[i, -(1:3)])) +
    rnorm(nsim, mean = 0, sd = attr(VarCorr(m_v), "sc"))) %>%
    data.frame(1:nsim, s, p, .)
  names(tmp) <- c('simid_v', 'state', 'party', 'voteshare_pred')
  predictions_v <- bind_rows(predictions_v, tmp)
}
predictions_v <- tbl_df(predictions_v)

predictions_v %>% group_by(state, party) %>%
  summarise(mean = mean(voteshare_pred),
            lwr = quantile(voteshare_pred, .025),
            upr = quantile(voteshare_pred, .975))


# Unweighted model
# Vote shares (federal level) --------------------------------------------------

# merge predictions of turnout and predictions of party vote shares
predictions <- left_join(predictions_v, predictions_t, by = c('simid_v' = 'simid_t', 'state'))

# calculate the number of votes received by a party
predictions$votes_hat = predictions$voteshare_pred/100 * predictions$valid
#replace votes_hat_w = voteshare_hat_w/100 * valid if state != "Bundesgebiet"  // weighted model

predictions$simid <-
  paste(predictions$simid_t, predictions$simid_v, sep = '-')

# Aggregate to the national level
simdata <- predictions %>% group_by(simid, party) %>%
  summarise(votes_hat_party = sum(votes_hat)) %>%
  group_by(simid) %>%
  mutate(votes_hat_national = sum(votes_hat_party),
         voteshare_hat = votes_hat_party / votes_hat_national * 100)

confint <- simdata %>% group_by(party) %>%
  summarise(mean = round(mean(voteshare_hat), 1),
            lwr = round(quantile(voteshare_hat, .025), 1),
            upr = round(quantile(voteshare_hat, .975), 1))

write.csv(confint, 'confint_unweighted.csv',
          row.names = F)

# Weighted model
# Vote shares (state level) ----------------------------------------------------
d17 <- read.dta('master_reduced.dta') %>% tbl_df()
d17 <- d17 %>% arrange(state, party)

predictions_v_w <- tbl_df(data.frame())
for(i in 1:nrow(d17)) {
  tmp <- rnorm(nsim, d17$voteshare_hat[i], d17$voteshare_se[i]) +
    rnorm(nsim, mean = 0, sd = 5.764081)
  tmp <- data.frame(d17$state[i], d17$party[i], tmp)
  names(tmp) <- c('state', 'party', 'voteshare_hat')
  predictions_v_w <- bind_rows(predictions_v_w, tmp)
}

predictions_v_w$simid_v_w <- rep(1:nsim, nrow(d17))

predictions_v_w %>% arrange(simid_v_w)

# Weighted model
# Vote shares (federal level) --------------------------------------------------

# merge predictions of turnout and predictions of party vote shares
predictions_w <- left_join(predictions_v_w, predictions_t,
                           by = c('simid_v_w' = 'simid_t', 'state'))

# calculate the number of votes received by a party
predictions_w$votes_hat = predictions_w$voteshare_hat/100 * predictions_w$valid
#replace votes_hat_w = voteshare_hat_w/100 * valid if state != "Bundesgebiet"  // weighted model

predictions_w$simid <- predictions_w$simid_v_w

# Aggregate to the national level
simdata_w <- predictions_w %>% group_by(simid, party) %>%
  summarise(votes_hat_party = sum(votes_hat)) %>%
  group_by(simid) %>%
  mutate(votes_hat_national = sum(votes_hat_party),
         voteshare_hat = votes_hat_party / votes_hat_national * 100)

confint_weighted <-
simdata_w %>% group_by(party) %>%
  summarise(mean = round(mean(voteshare_hat), 1),
            lwr = round(quantile(voteshare_hat, .025), 1),
            upr = round(quantile(voteshare_hat, .975), 1))

write.csv(confint_weighted, 'confint_weighted.csv',
          row.names = F)